@@ -24,10 +24,12 @@ class Doist(tyming.Tymist):
     .tick method.
     The doist may treat .tyme as artificial time or synchonize it to real time.
 
-    .ready method prepares dogs (doer generators) by calling generator functions
-        or generator methods and assigning to dogs list of tuples.
+    .ready method prepares deeds deque of triples (dog, retyme, index) where
+        dog is a doer generator returned by calling doer generator instances,
+        functions, or methods.
 
-    .once method runs its dogs (doer generators) once per invocation.
+    .once method runs its deeds deque of triples (dog, retyme, index) once per
+        invocation.
         This synchronizes their cycle time .tyme to the Doist's tyme.
 
 
@@ -53,9 +55,10 @@ class Doist(tyming.Tymist):
         .tick increments .tyme by one .tock or provided tock
 
     Methods:
-        .ready prepare doer generators (dogs)
-        .once  run through dogs one time
-        .do repeadedly call .once until all dogs are complete or times out
+        .ready prepare deeds, deque of triples (dog, retyme, index)
+        .once  run through all deeds once
+        .do repeadedly call .once until all dogs in deeds are complete or
+            times out do to reaching time limit
 
     """
     def __init__(self, real=False, limit=None, doers=None, **kwa):
@@ -81,14 +84,14 @@ class Doist(tyming.Tymist):
 
     def ready(self, doers=None):
         """
-        Returns dogs deque of triples (dog, retyme, index)  where:
+        Returns deeds deque of triples (dog, retyme, index)  where:
             dog is generator
             retyme is tyme (real or simulated) in seconds when dog should run next
             index is position of associated doer in .doers list used to assign
                 .done state to associated doer for dog
 
-        Calls each generator callable (function or method) in .doers to create
-        each generator dog.
+        Calls each generator callable (instance or function or method) in .doers
+        to create each generator dog.
 
         Runs enter context of each dog by calling next(dog)
 
@@ -101,10 +104,10 @@ class Doist(tyming.Tymist):
         See: https://stackoverflow.com/questions/40528867/setting-attributes-on-func
         For setting attributes on bound methods.
         """
-        if doers is not None:
-            self.doers = doers
+        if doers is None:
+            doers = self.doers
 
-        dogs = deque()
+        deeds = deque()
         for index, doer in enumerate(self.doers):
             if ismethod(doer):  # when using bound method for generator function
                 doer.__func__.done = None  # None before enter. enter may set to False
@@ -116,24 +119,24 @@ class Doist(tyming.Tymist):
                 next(dog)  # run enter by advancing to first yield
             except StopIteration:
                 continue  # don't append
-            dogs.append((dog, self.tyme, index))
-        return dogs
+            deeds.append((dog, self.tyme, index))
+        return deeds
 
 
-    def once(self, dogs):
+    def once(self, deeds):
         """
-        Cycle once through dogs deque and update in place
-        dogs is deque of triples of (dog, retyme, index) where:
+        Cycle once through deeds deque of triples and update in place
+        deeds is deque of triples of (dog, retyme, index) where:
             dog is generator
             retyme is tyme (real or simulated) in seconds when dog should run next
             index is position of associated doer in .doers list used to assign
                 .done state to associated doer for dog
 
-        Each cycle checks all generators in dogs deque and runs if retyme past.
+        Each cycle checks all generators in deeds deque and runs if retyme past.
         At end of cycle advances .tyme by one .tock by calling .tick()
         """
-        for i in range(len(dogs)):  # iterate once over each deed
-            dog, retyme, index = dogs.popleft()  # pop it off
+        for i in range(len(deeds)):  # iterate once over each deed
+            dog, retyme, index = deeds.popleft()  # pop it off
 
             if retyme <= self.tyme:  # run it now
                 try:
@@ -143,24 +146,24 @@ class Doist(tyming.Tymist):
                 else:  # reappend for next pass
                     if tock is None:  # bare yield returns None
                         tock = 0.0  # rerun asap when tock == 0.0
-                    dogs.append((dog, retyme + tock, index))  # tock may change during run
+                    deeds.append((dog, retyme + tock, index))  # tock may change during run
             else:  # not retyme yet
-                dogs.append((dog, retyme, index))  # reappend for next pass
+                deeds.append((dog, retyme, index))  # reappend for next pass
 
         self.tick()  # advance .tyme by one doist .tock
 
 
     def do(self, doers=None, limit=None, tyme=None):
         """
-        Readies dogs deque from .doers or doers if any and then runs .once with
-        dogs until completion
-        Each entry in dogs is triples (dog, retyme, index)  where:
+        Readies deeds deque from .doers or doers if any and then iteratively
+        runs .once over deeds deque until completion of all deeds.
+        Each entry in deeds is a triple (dog, retyme, index)  where:
             dog is generator
             retyme is tyme (real or simulated) in seconds when dog should run next
             index is position of associated doer in .doers list used to assign
                 .done state to associated doer for dog
 
-        If interrupted by exception call .close on each dog to force  exit context.
+        If interrupted by exception call .close on each dog to force exit context.
 
         Keyboard interrupt (cntl-c) forces exit.
 
@@ -170,20 +173,23 @@ class Doist(tyming.Tymist):
         See: https://stackoverflow.com/questions/40528867/setting-attributes-on-func
         For setting attributes on bound methods.
         """
+        if doers is not None:
+            self.doers = doers
+
         if limit is not None:  # time limt for running if any. useful in test
             self.limit = abs(float(limit))
 
         if tyme is not None:  # re-initialize starting tyme
             self.tyme = tyme
 
-        dogs = self.ready(doers=doers)  # runs enter context on each doer
+        deeds = self.ready(doers=doers)  # runs enter context on each doer
 
         tymer = tyming.Tymer(tymist=self, duration=self.limit)
         self.timer.start()
         try:  # always clean up resources upon exception
             while True:  # until doers complete or exception or keyboardInterrupt
                 try:
-                    self.once(dogs)  # increments .tyme runs recur context
+                    self.once(deeds)  # increments .tyme runs recur context
 
                     if self.real:  # wait for real time to expire
                         while not self.timer.expired:
@@ -193,7 +199,7 @@ class Doist(tyming.Tymist):
                     if self.limit and tymer.expired:
                         break  # break out of forever loop
 
-                    if not dogs:  # no more remaining dogs so done
+                    if not deeds:  # no more remaining deeds so done
                         break  # break out of forever loop
 
                 except KeyboardInterrupt:  # use CNTL-C to shutdown from shell
@@ -206,26 +212,41 @@ class Doist(tyming.Tymist):
                     raise
 
 
-        finally: # finally clause always runs regardless of exception or not
-            # exit in each dog is run by try finally clause. Each dogs exit is
-            # responsible for releasing resources
-            # Previously aborted or closed dogs have already exited
-            # Close any running dogs in reverse order to force exit and reclaim
-            # resources. enters and exits are nested pairs in reverse order so
-            # nested resource dependencies are maintained.
-            #  enter A, enter B, enter C, exit C, exit B, exit A
-            while(dogs):  # .close each remaining do in dogs in reverse order
-                dog, retime, index = dogs.pop() #pop it off
-                try:
-                    tock = dog.close()  # force GeneratorExit. Maybe log exit tock tyme
-                except StopIteration:
-                    pass  # Hmm? Not supposed to happen!
-                else:  # set done state
-                    doer = self.doers[index]
-                    if ismethod(doer):  # when using bound method for generator function
-                        doer.__func__.done = False  # forced close
-                    else:
-                        doer.done = False  # forced close
+        finally: # finally clause always runs regardless of exception or not.
+            self.close(deeds)  # force close remaining deeds throws GeneratorExit
+
+
+    def close(self, deeds):
+        """
+        Force exit each still opened deed calling .close on the dog generator
+        which throws a GeneratorExit to the generator.
+        This executes the close context (GeneratorExit) which then excecutes
+        the exit context in the finally caluse. Each dogs exit is responsible
+        for releasing resources
+        Previously aborted or closed dogs have already exited
+        Close any running dogs in reverse order so that enters and exits are
+        nested pairs so that the corresponding exits appear in reverse order
+        to their entes. This preserves nested resource dependencies.
+        For example:
+            enter A,
+                enter B,
+                    enter C,
+                    exit C,
+                exit B,
+            exit A
+        """
+        while(deeds):  # .close each remaining dog in deeds in reverse order
+            dog, retime, index = deeds.pop() #pop it off
+            try:
+                tock = dog.close()  # force GeneratorExit. Maybe log exit tock tyme
+            except StopIteration:
+                pass  # Hmm? Not supposed to happen!
+            else:  # set done state
+                doer = self.doers[index]
+                if ismethod(doer):  # when using bound method for generator function
+                    doer.__func__.done = False  # forced close
+                else:
+                    doer.done = False  # forced close
 
 
 def doify(f, name=None, tock=0.0, **opts):
@@ -382,7 +403,7 @@ class Doer(tyming.Tymee):
         0.0 means run asap,
         set ._tock to tock
         """
-        self._tock= abs(float(tock))
+        self._tock = abs(float(tock))
 
 
     def do(self, tymist=None, tock=0.0, **opts):
@@ -593,6 +614,15 @@ class DoDoer(Doer):
         .tock is float, desired time in seconds between runs or until next run,
                  non negative, zero means run asap
 
+    Properties:
+        .deeds is deque of triples (dog, retyme, index)  where:
+            dog is generator
+            retyme is tyme in seconds when next should run may be real or simulated
+            index is position of associated doer in .doers list
+        .always is Boolean, True means keep running even when all dogs in deeds
+            are complete. Enables dynamically managing extending or removing
+            doers and associated deeds while running.
+
     Inherited Methods:
         .wind  injects ._tymist dependency
         .__call__ makes instance callable
@@ -614,48 +644,101 @@ class DoDoer(Doer):
     Hidden:
        ._tymist is Tymist instance reference
        ._tock is hidden attribute for .tock property
+       ._always is hidden attribute for .always propery
 
     """
 
-    def __init__(self, doers=None, **kwa):
+    def __init__(self, doers=None, always=False, **kwa):
         """
         Initialize instance.
 
         Inherited Parameters:
             tymist is  Tymist instance
+            tock is float seconds initial value of .tock
 
         Parameters:
-           tock is float seconds initial value of .tock
+           doers is list of doer compatible generator functions
+           always is Boolean, True means keep running even when all dogs in deeds
+            are complete. Enables dynamically managing extending or removing
+            doers and associated deeds while running.
 
         """
         super(DoDoer, self).__init__(**kwa)
         self.doers = doers if doers is not None else []
+        self.always = always
+        self.deeds = deque()
 
 
-    def do(self, tymist=None, tock=0.0, doers=None, **opts):
+    @property
+    def deeds(self):
         """
-        Generator method to run this doer
+        deeds property getter, get ._deeds
+        .deeds is deque of triples, each of form (dog, retyme, index)
+        """
+        return self._deeds
+
+
+    @deeds.setter
+    def deeds(self, deeds):
+        """
+        set ._deeds to deeds deque
+        """
+        if not isinstance(deeds, deque):
+            raise TypeError("Expected deque, got {}.".format(type(deeds)))
+        self._deeds = deeds
+
+
+    @property
+    def always(self):
+        """
+        always property getter, get ._always
+        .always is Boolean, True means keep running even when all dogs in deeds
+            are complete. Enables dynamically managing extending or removing
+            doers and associated deeds while running.
+        """
+        return self._always
+
+
+    @always.setter
+    def always(self, always):
+        """
+        set ._always to always
+        """
+        self._always = True if always else False
+
+
+    def do(self, tymist=None, tock=0.0, doers=None, always=None, **opts):
+        """
+        Generator method to run this doer. Equivalent of doist.do
         Calling this method returns generator
         Interface matched generator function for compatibility
 
         Parameters:
-            tymist is injected Tymist instance with tymist.tyme
+            tymist is injected Tymist instance with tymist.tyme. So tymist
+                same as doist or dodoer that is doing this dodoer.
             tock is injected initial tock value
             opts is dict of injected optional additional parameters
+            doers is list of doers (generator functions)
+            always is Boolean. True means keep running even when all dogs in deeds
+                are complete. Enables dynamically managing extending or removing
+                doers and associated deeds while running.
+                When not provided use .always.
         """
-        dogs = deque()
+        always = always if always is not None else self.always
+        if doers is not None:
+            self.doers = doers
         try:
             # enter context
             self.wind(tymist)  # change tymist and dependencies
             self.tock = tock  #  set tock to parameter
-            tyme = self.tyme
+            # tyme = self.tyme
             self.done = False  # allows enter to override completion state
-            dogs = self.enter(doers=doers)
+            self.deeds = self.enter(doers=self.doers)  # doist.ready equivalent
 
             #recur context
-            while (not self.done):  # recur context
+            while (not self.done or always):  # recur context
                 tyme = (yield (self.tock))  # yields .tock then waits for next send
-                self.done = self.recur(tyme=tyme, dogs=dogs)
+                self.done = self.recur(tyme=tyme, deeds=self.deeds)  # equv of doist.once
 
         except GeneratorExit:  # close context, forced exit due to .close
             self.close()
@@ -668,17 +751,17 @@ class DoDoer(Doer):
             self.clean()
 
         finally:  # exit context, exit, unforced if normal exit of try, forced otherwise
-            self.exit(dogs=dogs)
+            self.exit(deeds=self.deeds, doers=self.doers)  # equiv of doist.do finally clause
 
         # return value of yield from or StopIteration.value indicates completion
         return self.done  # Only returns done state if normal return not close or abort raise
 
 
-    def enter(self, doers=None):
+    def enter(self, doers=None, deeds=None):
         """
         Do 'enter' context actions. Equivalent of Doist.ready()
 
-         Returns dogs deque of triples (dog, retyme, index)  where:
+        Returns deeds deque of triples (dog, retyme, index)  where:
             dog is generator
             retyme is tyme in seconds when next should run may be real or simulated
             index is position of associated doer in .doers list
@@ -690,16 +773,19 @@ class DoDoer(Doer):
 
         Parameters:
             doers is list of generator method or function callables with attributes
-                tock, done, and opts dict()
+                tock, done, and opts dict(). Use .doers if not provided
+            deeds is deque of deed triples of form (dog, retyme, index).
+                Use .deeds if not provided
 
         See: https://stackoverflow.com/questions/40528867/setting-attributes-on-func
         For setting attributes on bound methods.
         """
-        if doers is not None:
-            self.doers = doers
+        if doers is None:
+            doers = self.doers
+        if deeds is None:
+            deeds = self.deeds
 
-        dogs = deque()
-        for index, doer in enumerate(self.doers):
+        for index, doer in enumerate(doers):
             if ismethod(doer):  # when using bound method for generator function
                 doer.__func__.done = None  # None before enter. enter may set to False
             else:
@@ -710,70 +796,143 @@ class DoDoer(Doer):
                 next(dog)  # run enter by advancing to first yield
             except StopIteration:
                 continue  # don't append
-            dogs.append((dog, self.tyme, index))
-        return dogs
+            deeds.append((dog, self.tyme, index))
+        return deeds
 
 
-    def recur(self, tyme, dogs):
+    def recur(self, tyme, deeds=None):
         """
-        Do 'recur' context actions.
+        Do 'recur' context actions. Equivalent of Doist.once
 
         Parameters:
             tyme is output of send fed to do yield, Doist feeds its .tyme
+                because tymist is injected by doist or dodoer doing this dodoer
+                self.tyme is same as tyme.
+            deeds is deque of triples of (dog, retyme, index) where:
+                dog is generator
+                retyme is tyme in seconds when next should run may be real or simulated
+                index is position of associated doer in .doers list
 
         Returns completion state of recurrence actions.
            True means done False means continue
 
-        Cycle once through dogs deque and update in place
-        dogs is deque of triples of (dog, retyme, index) where:
-            dog is generator
-            retyme is tyme in seconds when next should run may be real or simulated
-            index is position of associated doer in .doers list
+        Cycle once through deeds deque and update in place
 
-        Each cycle checks all generators in dogs deque and runs if retyme past.
+        Each cycle checks all generators dogs in deeds deque and runs if retyme past.
         """
-        for i in range(len(dogs)):  # iterate once over each deed
-            dog, retyme, index = dogs.popleft()  # pop it off
+        if deeds is None:
+            deeds = self.deeds
 
-            if retyme <= self.tyme:  # run it now
+        for i in range(len(deeds)):  # iterate once over each deed
+            dog, retyme, index = deeds.popleft()  # pop it off
+
+            if retyme <= tyme:  # run it now
                 try:  # tock == 0.0 means re-run asap
-                    tock = dog.send(self.tyme)  # send tyme. yield tock
+                    tock = dog.send(tyme)  # send tyme. yield tock
                 except StopIteration as ex:  # returned instead of yielded
                     self.doers[index].done = ex.value if ex.value else False  # assign done state
                 else:  # reappend for next pass
                     if tock is None:  # bare yield results None
                         tock = 0.0  # # rerun asap when tock == 0.0
-                    dogs.append((dog, retyme + tock, index))  # tock may change during run
+                    deeds.append((dog, retyme + tock, index))  # tock may change during run
             else:  # not retyme yet
-                dogs.append((dog, retyme, index))  # reappend for next pass
+                deeds.append((dog, retyme, index))  # reappend for next pass
 
-        return (not dogs)  # True if dogs empty
+        return (not deeds)  # True if deeds deque is empty
 
 
-    def exit(self, dogs):
+    def exit(self, deeds=None, doers=None):
         """
         Do 'exit' context actions.
 
         Parameters:
-            dogs is deque of duples (dog, retyme)
+            deeds is deque of triples (dog, retyme, index)
+            doers is list of doers used to generate deeds, i.e. index
+                for a given deed triple must match index in doers list
 
         See: https://stackoverflow.com/questions/40528867/setting-attributes-on-func
         For setting attributes on bound methods.
         """
-        while(dogs):  # .close each remaining do in dogs in reverse order
-            dog, retime, index = dogs.pop() #pop it off
+        if deeds is None:
+            deeds = self.deeds
+        if doers is None:
+            doers = self.doers
+
+        while(deeds):  # .close each remaining dog in deeds in reverse order
+            dog, retime, index = deeds.pop()  # pop it off in reverse (right side)
             try:
                 tock = dog.close()  # force GeneratorExit
             except StopIteration:
                 pass  # Hmm? Not supposed to happen!
             else:  # set done state
-                doer = self.doers[index]
+                doer = doers[index]
                 if ismethod(doer):  # when using bound method for generator function
                     doer.__func__.done = False  # forced close
                 else:
                     doer.done = False  # forced close
 
 
+    def extend(self, doers):
+        """
+        Extend .doers list with doers. Ready deeds from doers and extend .doers
+        and .deeds.  Edit deeds in place so not replace deque.
+
+        Parameters:
+            doers is list of doers to add as extension.
+
+        """
+        deeds = self.enter(doers=doers, deeds=deque())  # provide fresh deeds
+        offset = len(self.doers)  # get offset of index for extending dogs
+        for i in range(len(deeds)):  # iterate once over each deed
+            dog, retyme, index = deeds.popleft()
+            index += offset
+            deeds.append((dog, retyme, index))
+
+        self.doers.extend(doers)
+        self.deeds.extend(deeds)
+
+
+    def remove(self, doers):
+        """
+        Remove doers from .doers list and any associated deeds from .deeds deque.
+        Force close removed deeds.
+
+        Parameters:
+            doers is list of doers to remove.
+
+        """
+        rdoers = list(doers)  # doers to remove, make copy since destructive
+        indices = {}  # map indices for each doer as appears in .doers to rdoers
+        j = 0  # index in remove list
+        for doer in list(rdoers):  # make copy since inplace filter of rdoers
+            try:
+                i = self.doers.index(doer)  # make sure a member of .doers
+            except ValueError:
+                rdoers.remove(doer)  # not in .doers so ignore remove from rdoers
+            else:
+                indices[i] = j
+                j += 1
+
+
+        rdeeds = deque()  # fresh deque for deeds to remove
+        deeds = self.deeds  # edit update self.deeds in place
+        for i in range(len(deeds)):  # iterate once over each deed
+            dog, retyme, index = deeds.popleft()
+            if index in indices:  # found deed to remove and close
+                rdeeds.append((dog, retyme, indices[index]))  # add to removal deque
+            else:  # keep deed do not remove and close
+                deeds.append((dog, retyme, index))  # reappend
+
+        for doer in rdoers:  # update .doers to remove rdoers
+            self.doers.remove(doer)
+
+        self.exit(deeds=rdeeds, doers=rdoers)
+
+
+
+
+
+
 
 class ServerDoer(Doer):
     """
